//Month Frame

//Copyright 2001 Tomas Koutny. Version 1.1; Revision 1
//The files MonthFrame.* are freeware and no one is liable for any damage
//that can be done by using it. You agree with it by using these files.

//For more info see
// http://www.rawos.com/qlinks/mf.htm
// http://www.rawos.com/qlinks/ec.htm
// http://www.rawos.com/sw


//Version 1.1
//      - added BeforeMonthChange event
//      - added OnMonthChange event
//      - added function MinDate
//      - added function DisplayedDays
//      - added property Data
//      - renamed property TodayColor to SelectedDayColor 
//      - signs of selected day and today were exchanged

unit MonthFrame;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCalendar, ExtCtrls, StdCtrls, ImgList, ToolWin, ComCtrls;

type
  TBeforeMonthChange = procedure(Sender:TObject; var CanChange:boolean) of object;
  TOnMonthChange = procedure(Sender:TObject) of object;

  TMonthFrame = class(TFrame)
    ExtCalendar: TExtCalendar;
    Panel1: TPanel;
    Bevel1: TBevel;
    TodayLabel: TLabel;
    TodayValueLabel: TLabel;
    DateTimePicker: TDateTimePicker;
    Label1: TLabel;
    TodayImage: TImage;
    procedure ExtCalendarDaySelect(ADay: TDate; AFlags: Word;
      var CanSelect: Boolean);
    procedure DateTimePickerChange(Sender: TObject);
    procedure ExtCalendarDayGroundDrawn(ADay: TDate; AFlags: Word;
      Canvas: TCanvas; ARect: TRect);
    procedure ExtCalendarDayDrawn(ADay: TDate; AFlags: Word;
      Canvas: TCanvas; ARect: TRect);
  private
    FChangingPickerDate,                //Semaphor for BeforeMonthChange
    FSelectingDay:boolean;              //Semaphor for OnDaySelect
    FLastToday:TDate;                   //Day we remeber as today
    FLastDate:TDate;                    //The last selected date
    FDaySel:TDaySelect;
    FOnDayDrawn:TDayDrawn;      
    FDayBckDrwn:TDayGroundDrawn;
  protected
    FSelColor:TColor;                 //Color of today
    FBeforeMonthChange:TBeforeMonthChange;
    FOnMonthChange:TOnMonthChange;
    procedure SetSelCol(AVal:TColor);
    function GetDateMask:string;
    procedure SetDateMask(AVal:String);
    function GetHeaderFont:TFont;
    procedure SetHeaderFont(AVal:TFont);
    function GetBodyFont:TFont;
    procedure SetBodyFont(AVal:TFont);
    function GetFooterFont:TFont;
    procedure SetFooterFont(AVal:TFont);
    function GetWeekStart:TCalDayOfWeek;
    procedure SetWeekStart(AVal:TCalDayOfWeek);
    function GetMDC:TColor;
    procedure SetMDC(AVal:TColor);
    function GetNMDC:TColor;
    procedure SetNMDC(AVal:TColor);
    function GetFDG:TColor;
    procedure SetFDG(AVal:TColor);
    function GetWDG:TColor;
    procedure SetWDG(AVal:TColor);
    function GetDayDraw:TDayDraw;
    procedure SetDayDraw(AVal:TDayDraw);
    function GetDayInfo:TDayGetInfo;
    procedure SetGetDayInfo(AVal:TDayGetInfo);
    function GetDay:TDate;
    procedure SetDay(AVal:TDate);
    function GetData(Index:integer):pointer;
    procedure SetData(Index:integer; AData:pointer);
    function GetKeyPress:TKeyPressEvent;
    procedure SetKeyPress(AVal:TKeyPressEvent);
  public
    constructor Create(AOwner:TComponent); override;

    procedure Repaint; override;        //Way how to refresh ExtCalendar
                                        //on a demand

    procedure RefreshToday;             //Refreshes today value
    function MinDate:TDate;
    function DisplayedDays:integer;
    property Data[index:integer]:pointer read GetData write SetData;
        //!!!Its dimesion is being changed immediately
        //   with the count of displayed days!!!
  published
    property DateMask:String read GetDateMask write SetDateMask;
    property HeaderFont:TFont read GetHeaderFont write SetHeaderFont;
    property BodyFont:TFont read GetBodyFont write SetBodyFont;
    property FooterFont:TFont read GetFooterFont write SetFooterFont;
    property WeekStart:TCalDayOfWeek read GetWeekStart write SetWeekStart;
    property SelectedDayColor:TColor read FSelColor write SetSelCol;
    property MonthDayColor:TColor read GetMDC write SetMDC;
    property NonMonthDayColor:TColor read GetNMDC write SetNMDC;
    property FreeDayGround:TColor read GetFDG write SetFDG;
    property WorkDayGround:TColor read GetWDG write SetWDG;
    property Day:TDate read GetDay write SetDay;

    property OnDaySelect:TDaySelect read FDaySel write FDaySel;
    property OnDayDraw:TDayDraw read GetDayDraw write SetDayDraw;
    property OnDayDrawn:TDayDrawn read FOnDayDrawn write FOnDayDrawn;
    property OnGetDayInfo:TDayGetInfo read GetDayInfo write SetGetDayInfo;
    property OnDayGroundDrawn:TDayGroundDrawn read FDayBckDrwn write FDayBckDrwn;
    property BeforeMonthChange:TBeforeMonthChange read FBeforeMonthChange
                                                  write FBeforeMonthChange;
    property OnMonthChange:TOnMonthChange read FOnMonthChange
                                          write FOnMonthChange;
    property OnKeyPress:TKeyPressEvent read GetKeyPress write SetKeyPress;
  end;

procedure Register;

implementation

{$R *.DFM}
{$R MonthFrame.DCR}

procedure Register;
begin
  RegisterComponents('XDTK', [TMonthFrame]);
end;


constructor TMonthFrame.Create(AOwner:TComponent);
begin
  inherited;

  RefreshToday;
  FSelectingDay:=false;
  FSelColor:=$00E08F00;
  FLastDate:=DateTimePicker.DateTime;
  with TodayLabel do
    TodayValueLabel.Left:=Width+Left;
end;


procedure TMonthFrame.RefreshToday;
begin
  FLastToday:=Date;
  TodayValueLabel.Caption:=FormatDateTime('', FLastToday);
end;

procedure TMonthFrame.ExtCalendarDaySelect(ADay: TDate; AFlags: Word;
  var CanSelect: Boolean);
begin
  if FSelectingDay then exit;
  FSelectingDay:=true;

  if Assigned(FDaySel) then FDaySel(ADay, AFlags, CanSelect);
  if CanSelect then DateTimePicker.DateTime:=ADay;
  FLastDate:=DateTimePicker.DateTime; //Rememeber currently selected date

  FSelectingDay:=false;
end;

procedure TMonthFrame.DateTimePickerChange(Sender: TObject);
var allow:boolean;
    OldYear, OldMonth, NewYear, NewMonth, XDay:word; //X dure to the property
begin
  if FSelectingDay or FChangingPickerDate then exit;

  //Presume allowance to change date
  allow:=true;

  with DateTimePicker do
    begin
      //Test the month change - if happened
      DecodeDate(FLastDate, OldYear, OldMonth, XDay);
      DecodeDate(DateTime, NewYear, NewMonth, XDay);

      if (OldMonth<>NewMonth) or (OldYear<>NewYear) then
        begin//A month has been changed =>
             //Ask user if the month change is allowed
         if Assigned(FBeforeMonthChange) then FBeforeMonthChange(Self, allow);
         if not allow then
           begin       //No change is allowed
             FChangingPickerDate:=true;
             DateTimePicker.DateTime:=FLastDate;
             FChangingPickerDate:=false;
           end;
        end;

      //if the month change is approved => ask for the day selection approve
      if allow and Assigned(FDaySel) then
        FDaySel(DateTime, ExtCalendar.GetDayInfo(DateTime), allow);

      if not allow then
         begin       //No change is allowed
           FChangingPickerDate:=true;
           DateTimePicker.DateTime:=FLastDate;
           FChangingPickerDate:=false;
         end;

      if allow then //The change of date is allowed
        begin
          FSelectingDay:=true;
          ExtCalendar.Day:=DateTime;
          FLastDate:=DateTime;
          FSelectingDay:=false;
          if Assigned(FOnMonthChange) then FOnMonthChange(Self);
        end;
     end;
end;

procedure TMonthFrame.ExtCalendarDayGroundDrawn(ADay: TDate; AFlags: Word;
  Canvas: TCanvas; ARect: TRect);
begin
  if Assigned(FDayBckDrwn) then FDayBckDrwn(ADay, AFlags, Canvas, ARect);
                                  
  if Int(ADay) = Int(ExtCalendar.Day) then
    with Canvas do
      begin
        Brush.Color:=FSelColor;
        FillRect(ARect);
      end;
end;

procedure TMonthFrame.SetSelCol(AVal:TColor);
begin
  FSelColor:=AVal;
  ExtCalendar.Refresh;
end;

function TMonthFrame.GetDateMask:string;
begin
  result:=ExtCalendar.Mask;
end;

procedure TMonthFrame.SetDateMask(AVal:String);
begin
  ExtCalendar.Mask:=AVal;
end;

function TMonthFrame.GetHeaderFont:TFont;
begin
  result:=ExtCalendar.HeaderFont;
end;

procedure TMonthFrame.SetHeaderFont(AVal:TFont);
begin
  ExtCalendar.HeaderFont:=AVal;
end;

function TMonthFrame.GetBodyFont:TFont;
begin
  result:=ExtCalendar.BodyFont;
end;

procedure TMonthFrame.SetBodyFont(AVal:TFont);
begin
  ExtCalendar.BodyFont:=AVal;
end;
function TMonthFrame.GetFooterFont:TFont;
begin
  result:=TodayValueLabel.Font;
end;

procedure TMonthFrame.SetFooterFont(AVal:TFont);
begin
  with TodayLabel do
    begin
      Font:=AVal;
      TodayValueLabel.Left:=Width+Left;
    end;
  TodayValueLabel.Font:=AVal;    
  DateTimePicker.Font:=AVal;
end;

function TMonthFrame.GetWeekStart:TCalDayOfWeek;
begin
  result:=ExtCalendar.WeekStart;
end;

procedure TMonthFrame.SetWeekStart(AVal:TCalDayOfWeek);
begin
  ExtCalendar.WeekStart:=AVal;
end;

function TMonthFrame.GetMDC:TColor;
begin
  result:=ExtCalendar.MonthDayColor;
end;

procedure TMonthFrame.SetMDC(AVal:TColor);
begin
  ExtCalendar.MonthDayColor:=AVal;
end;

function TMonthFrame.GetNMDC:TColor;
begin
  result:=ExtCalendar.NonMonthDayColor;
end;

procedure TMonthFrame.SetNMDC(AVal:TColor);
begin
  ExtCalendar.NonMonthDayColor:=AVal;
end;

function TMonthFrame.GetFDG:TColor;
begin
  result:=ExtCalendar.FreeDayGround;
end;

procedure TMonthFrame.SetFDG(AVal:TColor);
begin
  ExtCalendar.FreeDayGround:=AVal;
end;

function TMonthFrame.GetWDG:TColor;
begin
  result:=ExtCalendar.WorkDayGround;
end;

procedure TMonthFrame.SetWDG(AVal:TColor);
begin
  ExtCalendar.WorkDayGround:=AVal;
end;

function TMonthFrame.GetDayDraw:TDayDraw;
begin
  result:=ExtCalendar.OnDayDraw;
end;

procedure TMonthFrame.SetDayDraw(AVal:TDayDraw);
begin
  ExtCalendar.OnDayDraw:=AVal;
end;

function TMonthFrame.GetDayInfo:TDayGetInfo;
begin
  result:=ExtCalendar.OnGetDayInfo;
end;

procedure TMonthFrame.SetGetDayInfo(AVal:TDayGetInfo);
begin
  ExtCalendar.OnGetDayInfo:=AVal;
end;

function TMonthFrame.GetDay:TDate;
begin
  result:=ExtCalendar.Day;
end;

procedure TMonthFrame.SetDay(AVal:TDate);
begin
  ExtCalendar.Day:=AVal;
end;

function TMonthFrame.GetData(Index:integer):pointer;
begin
  result:=ExtCalendar.Data[Index];
end;

procedure TMonthFrame.SetData(Index:integer; AData:pointer);
begin
  ExtCalendar.Data[Index]:=AData;
end;

function TMonthFrame.MinDate:TDate;
begin
  result:=ExtCalendar.MinDate;
end;

function TMonthFrame.DisplayedDays:integer;
begin
  result:=ExtCalendar.DisplayedDays;
end;

procedure TMonthFrame.Repaint;
begin
  ExtCalendar.Refresh;

  inherited;
end;  

procedure TMonthFrame.ExtCalendarDayDrawn(ADay: TDate; AFlags: Word;
  Canvas: TCanvas; ARect: TRect);
var tmp:TDate;  
begin 
  tmp:=Date;
  if Int(ADay) = Int(tmp) then
    with Canvas do
      begin
        if tmp<>FLastToday then RefreshToday;

        Canvas.StretchDraw(ARect, TodayImage.Picture.Metafile);
      end;

  if Assigned(FOnDayDrawn) then FOnDayDrawn(ADay, AFlags, Canvas, ARect);
end;

function TMonthFrame.GetKeyPress:TKeyPressEvent;
begin
  result:=ExtCalendar.OnKeyPress;
end;

procedure TMonthFrame.SetKeyPress(AVal:TKeyPressEvent);
begin
  ExtCalendar.OnKeyPress:=AVal;
end;


end.
